@@ -127,7 +127,7 @@ class FactoidsCallback(httpserver.SupyHTTPServerCallback):
             self.end_headers()
             self.write(httpserver.get_template('factoids/index.html'))
         elif len(parts) == 2:
-            channel = utils.web.unquote(parts[0])
+            channel = utils.web.urlunquote(parts[0])
             if not ircutils.isChannel(channel):
                 self.send_response(404)
                 self.send_header('Content-type', 'text/html; charset=utf-8')
@@ -159,8 +159,7 @@ class FactoidsCallback(httpserver.SupyHTTPServerCallback):
                     factoids[key] = {}
                 factoids[key][id_] = fact
             content = ''
-            keys = factoids.keys()
-            keys.sort()
+            keys = sorted(factoids.keys())
             for key in keys:
                 facts = factoids[key]
                 content += '<tr>'
@@ -182,7 +181,7 @@ class FactoidsCallback(httpserver.SupyHTTPServerCallback):
         if 'chan' in form:
             self.send_response(303)
             self.send_header('Location',
-                    './%s/' % utils.web.quote(form['chan'].value))
+                    './%s/' % utils.web.urlquote(form['chan'].value))
             self.end_headers()
         else:
             self.send_response(400)
