@@ -42,6 +42,9 @@ import supybot.callbacks as callbacks
 from supybot.i18n import PluginInternationalization, internationalizeDocstring
 _ = PluginInternationalization('Later')
 
+class QueueIsFull(Exception):
+    pass
+
 class Later(callbacks.Plugin):
     """Used to do things later; currently, it only allows the sending of
     nick-based notes.  Do note (haha!) that these notes are *not* private
@@ -61,7 +64,7 @@ class Later(callbacks.Plugin):
     def _flushNotes(self):
         fd = utils.file.AtomicFile(self.filename)
         writer = csv.writer(fd)
-        for (nick, notes) in self._notes.iteritems():
+        for (nick, notes) in self._notes.items():
             for (time, whence, text) in notes:
                 writer.writerow([nick, time, whence, text])
         fd.close()
@@ -93,7 +96,7 @@ class Later(callbacks.Plugin):
         try:
             notes = self._notes[nick]
             if maximum and len(notes) >= maximum:
-                raise ValueError
+                raise QueueIsFull()
             else:
                 notes.append((at, whence, text))
         except KeyError:
@@ -125,7 +128,7 @@ class Later(callbacks.Plugin):
         expiry = self.registryValue('messageExpiry')
         curtime = time.time()
         nickremovals=[]
-        for (nick, notes) in self._notes.iteritems():
+        for (nick, notes) in self._notes.items():
             removals = []
             for (notetime, whence, text) in notes:
                 td = datetime.timedelta(seconds=(curtime - notetime))
@@ -147,27 +150,38 @@ class Later(callbacks.Plugin):
     ## adding new ones.
 
     @internationalizeDocstring
-    def tell(self, irc, msg, args, nick, text):
-        """<nick> <text>
+    def tell(self, irc, msg, args, nicks, text):
+        """<nick1[,nick2[,...]]> <text>
 
-        Tells <nick> <text> the next time <nick> is seen.  <nick> can
+        Tells each <nickX> <text> the next time <nickX> is seen.  <nickX> can
         contain wildcard characters, and the first matching nick will be
         given the note.
         """
         self._deleteExpired()
-        if ircutils.strEqual(nick, irc.nick):
-            irc.error(_('I can\'t send notes to myself.'))
-            return
-        validnick = self._validateNick(irc, nick)
-        if validnick is False:
-            irc.error('That is an invalid IRC nick. Please check your input.')
-            return
-        try:
-            self._addNote(validnick, msg.nick, text)
+        validnicks = []
+        for nick in nicks:
+            if ircutils.strEqual(nick, irc.nick):
+                irc.error(_('I can\'t send notes to myself.'))
+                return
+            validnick = self._validateNick(irc, nick)
+            if validnick is False:
+                irc.error(_('%s is an invalid IRC nick. Please check your '
+                    'input.' % nick))
+                return
+            validnicks.append(validnick)
+        full_queues = []
+        for validnick in validnicks:
+            try:
+                self._addNote(validnick, msg.nick, text)
+            except QueueIsFull:
+                full_queues.append(validnick)
+        if full_queues:
+            irc.error(format(
+                _('These recipients\' message queue are already full: %L'),
+                full_queues))
+        else:
             irc.replySuccess()
-        except ValueError:
-            irc.error(_('That person\'s message queue is already full.'))
-    tell = wrap(tell, ['something', 'text'])
+    tell = wrap(tell, [commalist('somethingWithoutSpaces'), 'text'])
 
     @internationalizeDocstring
     def notes(self, irc, msg, args, nick):
@@ -241,12 +255,14 @@ class Later(callbacks.Plugin):
             for removal in removals:
                 self.wildcards.remove(removal)
         if notes:
+            old_repliedto = msg.repliedTo
             irc = callbacks.SimpleProxy(irc, msg)
             private = self.registryValue('private')
             for (when, whence, note) in notes:
                 s = self._formatNote(when, whence, note)
                 irc.reply(s, private=private, prefixNick=not private)
             self._flushNotes()
+            msg.tag('repliedTo', old_repliedto)
 
     def _formatNote(self, when, whence, note):
         return _('Sent %s: <%s> %s') % (self._timestamp(when), whence, note)
