@@ -33,16 +33,13 @@ import io
 import sys
 import json
 import shutil
-import urllib
 import tarfile
-from cStringIO import StringIO
-
-BytesIO = StringIO if sys.version_info[0] < 3 else io.BytesIO
 
 import supybot.log as log
 import supybot.conf as conf
 import supybot.utils as utils
 from supybot.commands import *
+import supybot.utils.minisix as minisix
 import supybot.plugins as plugins
 import supybot.ircutils as ircutils
 import supybot.callbacks as callbacks
@@ -80,7 +77,7 @@ class GithubRepository(GitRepository):
     def _query(self, type_, uri_end, args={}):
         args = dict([(x,y) for x,y in args.items() if y is not None])
         url = '%s/%s/%s?%s' % (self._apiUrl, type_, uri_end,
-                               urllib.urlencode(args))
+                               utils.web.urlencode(args))
         return json.loads(utils.web.getUrl(url).decode('utf8'))
 
     def getPluginList(self):
@@ -104,11 +101,11 @@ class GithubRepository(GitRepository):
     def _download(self, plugin):
         try:
             response = utils.web.getUrlFd(self._downloadUrl)
-            if sys.version_info[0] < 3:
+            if minisix.PY2:
                 assert response.getcode() == 200, response.getcode()
             else:
                 assert response.status == 200, response.status
-            fileObject = BytesIO()
+            fileObject = minisix.io.BytesIO()
             fileObject.write(response.read())
         finally: # urllib does not handle 'with' statements :(
             response.close()
@@ -127,7 +124,7 @@ class GithubRepository(GitRepository):
             assert archive.getmember(prefix + dirname).isdir(), \
                 'This is not a valid plugin (it is a file, not a directory).'
 
-            run_2to3 = sys.version_info[0] >= 3
+            run_2to3 = minisix.PY3
             for file in archive.getmembers():
                 if file.name.startswith(prefix + dirname):
                     extractedFile = archive.extractfile(file)
@@ -144,7 +141,7 @@ class GithubRepository(GitRepository):
                         with open(newFileName, 'ab') as fd:
                             reload_imported = False
                             for line in extractedFile.readlines():
-                                if sys.version_info[0] >= 3:
+                                if minisix.PY3:
                                     if 'import reload' in line.decode():
                                         reload_imported = True
                                     elif not reload_imported and \
@@ -188,7 +185,7 @@ class GithubRepository(GitRepository):
             if file.name.startswith(prefix + dirname + '/README'):
                 extractedFile = archive.extractfile(file)
                 content = extractedFile.read()
-                if sys.version_info[0] >= 3:
+                if minisix.PY3:
                     content = content.decode()
                 return content
 
@@ -304,6 +301,14 @@ repositories = utils.InsensitivePreservingDict({
                                                    'waratte',
                                                    'supybot',
                                                    ),
+               't3chguy':          GithubRepository(
+                                                   't3chguy',
+                                                   'Limnoria-Plugins',
+                                                   ),
+                'prgmrbill':       GithubRepository(
+                                                   'prgmrbill',
+                                                   'limnoria-plugins',
+                                                   ),
                })
 
 class PluginDownloader(callbacks.Plugin):
