@@ -29,8 +29,6 @@
 
 import re
 import sys
-import uuid
-import time
 import fnmatch
 
 import supybot.conf as conf
@@ -82,7 +80,7 @@ class User(callbacks.Plugin):
                 return r.match(u.name) is not None
             predicates.append(p)
         users = []
-        for u in ircdb.users.itervalues():
+        for u in ircdb.users.values():
             for predicate in predicates:
                 if not predicate(u):
                     break
@@ -90,7 +88,8 @@ class User(callbacks.Plugin):
                 users.append(u.name)
         if users:
             utils.sortBy(str.lower, users)
-            irc.reply(format('%L', users))
+            private = self.registryValue("listInPrivate", msg.args[0])
+            irc.reply(format('%L', users), private=private)
         else:
             if predicates:
                 irc.reply(_('There are no matching registered users.'))
@@ -127,7 +126,7 @@ class User(callbacks.Plugin):
             if u._checkCapability('owner'):
                 addHostmask = False
             else:
-                irc.error(_('Your hostmask is already registered to %s') % 
+                irc.error(_('Your hostmask is already registered to %s') %
                           u.name)
                 return
         except KeyError:
@@ -197,10 +196,9 @@ class User(callbacks.Plugin):
             """[<name>] <old password> <new password>
 
             Sets the new password for the user specified by <name> to <new
-            password>.  Obviously this message must be sent to the bot
+            password>. Obviously this message must be sent to the bot
             privately (not in a channel). If the requesting user is an owner
-            user (and the user whose password is being changed isn't that same
-            owner user), then <old password> needn't be correct.
+            user, then <old password> needn't be correct.
             """
             try:
                 u = ircdb.users.getUser(msg.prefix)
@@ -211,7 +209,7 @@ class User(callbacks.Plugin):
                     irc.errorNotRegistered(Raise=True)
                 user = u
             if user.checkPassword(password) or \
-               (u and u._checkCapability('owner') and not u == user):
+               (u and u._checkCapability('owner')):
                 user.setPassword(newpassword)
                 ircdb.users.setUser(user)
                 irc.replySuccess()
@@ -410,140 +408,6 @@ class User(callbacks.Plugin):
         else:
             irc.error(_('GPG features are not enabled.'))
 
-    class gpg(callbacks.Commands):
-        def __init__(self, *args):
-            super(User.gpg, self).__init__(*args)
-            self._tokens = {}
-
-        def _expire_tokens(self):
-            now = time.time()
-            self._tokens = dict(filter(lambda x_y: x_y[1][1]>now,
-                self._tokens.items()))
-
-        @internationalizeDocstring
-        def add(self, irc, msg, args, user, keyid, keyserver):
-            """<key id> <key server>
-
-            Add a GPG key to your account."""
-            if keyid in user.gpgkeys:
-                irc.error(_('This key is already associated with your '
-                    'account.'))
-                return
-            result = gpg.keyring.recv_keys(keyserver, keyid)
-            reply = format(_('%n imported, %i unchanged, %i not imported.'),
-                    (result.imported, _('key')),
-                    result.unchanged,
-                    result.not_imported,
-                    [x['fingerprint'] for x in result.results])
-            if result.imported == 1:
-                user.gpgkeys.append(keyid)
-                irc.reply(reply)
-            else:
-                irc.error(reply)
-        add = wrap(add, ['user',
-                         ('somethingWithoutSpaces',
-                             _('You must give a valid key id')),
-                         ('somethingWithoutSpaces',
-                             _('You must give a valid key server'))])
-
-        @internationalizeDocstring
-        def remove(self, irc, msg, args, user, fingerprint):
-            """<fingerprint>
-
-            Remove a GPG key from your account."""
-            try:
-                keyids = [x['keyid'] for x in gpg.keyring.list_keys()
-                        if x['fingerprint'] == fingerprint]
-                if len(keyids) == 0:
-                    raise ValueError
-                for keyid in keyids:
-                    try:
-                        user.gpgkeys.remove(keyid)
-                    except ValueError:
-                        user.gpgkeys.remove('0x' + keyid)
-                gpg.keyring.delete_keys(fingerprint)
-                irc.replySuccess()
-            except ValueError:
-                irc.error(_('GPG key not associated with your account.'))
-        remove = wrap(remove, ['user', 'somethingWithoutSpaces'])
-
-        @internationalizeDocstring
-        def list(self, irc, msg, args, user):
-            """takes no arguments
-
-            List your GPG keys."""
-            keyids = user.gpgkeys
-            if len(keyids) == 0:
-                irc.reply(_('No key is associated with your account.'))
-            else:
-                irc.reply(format('%L', keyids))
-        list = wrap(list, ['user'])
-
-        @internationalizeDocstring
-        def gettoken(self, irc, msg, args):
-            """takes no arguments
-
-            Send you a token that you'll have to sign with your key."""
-            self._expire_tokens()
-            token = '{%s}' % str(uuid.uuid4())
-            lifetime = conf.supybot.plugins.User.gpg.TokenTimeout()
-            self._tokens.update({token: (msg.prefix, time.time()+lifetime)})
-            irc.reply(_('Your token is: %s. Please sign it with your '
-                'GPG key, paste it somewhere, and call the \'auth\' '
-                'command with the URL to the (raw) file containing the '
-                'signature.') % token)
-        gettoken = wrap(gettoken, [])
-
-        _auth_re = re.compile(r'-----BEGIN PGP SIGNED MESSAGE-----\r?\n'
-                r'Hash: .*\r?\n\r?\n'
-                r'\s*({[0-9a-z-]+})\s*\r?\n'
-                r'-----BEGIN PGP SIGNATURE-----\r?\n.*'
-                r'\r?\n-----END PGP SIGNATURE-----',
-                re.S)
-        
-        @internationalizeDocstring
-        def auth(self, irc, msg, args, url):
-            """<url>
-
-            Check the GPG signature at the <url> and authenticates you if
-            the key used is associated to a user."""
-            self._expire_tokens()
-            content = utils.web.getUrl(url)
-            if sys.version_info[0] >= 3 and isinstance(content, bytes):
-                content = content.decode()
-            match = self._auth_re.search(content)
-            if not match:
-                irc.error(_('Signature or token not found.'), Raise=True)
-            data = match.group(0)
-            token = match.group(1)
-            if token not in self._tokens:
-                irc.error(_('Unknown token. It may have expired before you '
-                    'submit it.'), Raise=True)
-            if self._tokens[token][0] != msg.prefix:
-                irc.error(_('Your hostname/nick changed in the process. '
-                    'Authentication aborted.'), Raise=True)
-            verified = gpg.keyring.verify(data)
-            if verified and verified.valid:
-                keyid = verified.pubkey_fingerprint[-16:]
-                prefix, expiry = self._tokens.pop(token)
-                found = False
-                for (id, user) in ircdb.users.items():
-                    if keyid in [x[-len(keyid):] for x in user.gpgkeys]:
-                        try:
-                            user.addAuth(msg.prefix)
-                        except ValueError:
-                            irc.error(_('Your secure flag is true and your '
-                                      'hostmask doesn\'t match any of your '
-                                      'known hostmasks.'), Raise=True)
-                        ircdb.users.setUser(user, flush=False)
-                        irc.reply(_('You are now authenticated as %s.') %
-                                user.name)
-                        return
-                irc.error(_('Unknown GPG key.'), Raise=True)
-            else:
-                irc.error(_('Signature could not be verified. Make sure '
-                    'this is a valid GPG signature and the URL is valid.'))
-        auth = wrap(auth, ['url'])
 
     @internationalizeDocstring
     def capabilities(self, irc, msg, args, user):
@@ -628,7 +492,7 @@ class User(callbacks.Plugin):
         owners = 0
         admins = 0
         hostmasks = 0
-        for user in ircdb.users.itervalues():
+        for user in ircdb.users.values():
             users += 1
             hostmasks += len(user.hostmasks)
             try:
