@@ -31,12 +31,11 @@
 import re
 import sys
 import socket
-import HTMLParser
-import htmlentitydefs
 
 import supybot.conf as conf
 import supybot.utils as utils
 from supybot.commands import *
+import supybot.utils.minisix as minisix
 import supybot.plugins as plugins
 import supybot.commands as commands
 import supybot.ircutils as ircutils
@@ -44,35 +43,40 @@ import supybot.callbacks as callbacks
 from supybot.i18n import PluginInternationalization, internationalizeDocstring
 _ = PluginInternationalization('Web')
 
-class Title(HTMLParser.HTMLParser):
-    entitydefs = htmlentitydefs.entitydefs.copy()
+if minisix.PY3:
+    from html.parser import HTMLParser
+    from html.entities import entitydefs
+else:
+    from HTMLParser import HTMLParser
+    from htmlentitydefs import entitydefs
+
+class Title(utils.web.HtmlToText):
+    entitydefs = entitydefs.copy()
     entitydefs['nbsp'] = ' '
-    entitydefs['apos'] = '\''
     def __init__(self):
         self.inTitle = False
-        self.title = ''
-        HTMLParser.HTMLParser.__init__(self)
+        self.inSvg = False
+        utils.web.HtmlToText.__init__(self)
+
+    @property
+    def inHtmlTitle(self):
+        return self.inTitle and not self.inSvg
 
     def handle_starttag(self, tag, attrs):
         if tag == 'title':
             self.inTitle = True
+        elif tag == 'svg':
+            self.inSvg = True
 
     def handle_endtag(self, tag):
         if tag == 'title':
             self.inTitle = False
+        elif tag == 'svg':
+            self.inSvg = False
 
-    def handle_data(self, data):
-        if self.inTitle:
-            self.title += data
-
-    def handle_entityref(self, name):
-        if self.inTitle:
-            if name in self.entitydefs:
-                self.title += self.entitydefs[name]
-
-    def handle_charref(self, name):
-        if self.inTitle:
-            self.title += unichr(int(name))
+    def append(self, data):
+        if self.inHtmlTitle:
+            super(Title, self).append(data)
 
 class DelayedIrc:
     def __init__(self, irc):
@@ -96,9 +100,9 @@ def fetch_sandbox(f):
     def newf(self, irc, *args):
         try:
             replies = commands.process(process, self, irc, *args,
-                    timeout=10, heap_size=1024*1024,
+                    timeout=10, heap_size=10*1024*1024,
                     pn=self.name(), cn=f.__name__)
-        except commands.ProcessTimeoutError:
+        except (commands.ProcessTimeoutError, MemoryError):
             raise utils.web.Error(_('Page is too big or the server took '
                     'too much time to answer the request.'))
         else:
@@ -121,6 +125,37 @@ class Web(callbacks.PluginRegexp):
     """Add the help for "@help Web" here."""
     regexps = ['titleSnarfer']
 
+    def noIgnore(self, irc, msg):
+        return not self.registryValue('checkIgnored', msg.args[0])
+
+    def getTitle(self, irc, url, raiseErrors):
+        size = conf.supybot.protocols.http.peekSize()
+        (target, text) = utils.web.getUrlTargetAndContent(url, size=size)
+        try:
+            text = text.decode(utils.web.getEncoding(text) or 'utf8',
+                    'replace')
+        except UnicodeDecodeError:
+            pass
+        parser = Title()
+        if minisix.PY3 and isinstance(text, bytes):
+            if raiseErrors:
+                irc.error(_('Could not guess the page\'s encoding. (Try '
+                        'installing python-charade.)'), Raise=True)
+            else:
+                return None
+        parser.feed(text)
+        parser.close()
+        title = utils.str.normalizeWhitespace(''.join(parser.data).strip())
+        if title:
+            return (target, title)
+        elif raiseErrors:
+            if len(text) < size:
+                irc.error(_('That URL appears to have no HTML title.'),
+                        Raise=True)
+            else:
+                irc.error(format(_('That URL appears to have no HTML title '
+                                 'within the first %S.'), size), Raise=True)
+
     @fetch_sandbox
     def titleSnarfer(self, irc, msg, match):
         channel = msg.args[0]
@@ -130,39 +165,20 @@ class Web(callbacks.PluginRegexp):
             return
         if self.registryValue('titleSnarfer', channel):
             url = match.group(0)
+            if not self._checkURLWhitelist(url):
+                return
             r = self.registryValue('nonSnarfingRegexp', channel)
             if r and r.search(url):
                 self.log.debug('Not titleSnarfing %q.', url)
                 return
-            try:
-                size = conf.supybot.protocols.http.peekSize()
-                fd = utils.web.getUrlFd(url)
-                text = fd.read(size)
-                fd.close()
-            except socket.timeout as e:
-                self.log.info('Couldn\'t snarf title of %u: %s.', url, e)
-                if self.registryValue('snarferReportIOExceptions', channel):
-                     irc.reply(url+" : "+utils.web.TIMED_OUT, prefixNick=False)
+            r = self.getTitle(irc, url, False)
+            if not r:
                 return
-            try:
-                text = text.decode(utils.web.getEncoding(text) or 'utf8',
-                        'replace')
-            except:
-                pass
-            parser = Title()
-            try:
-                parser.feed(text)
-            except HTMLParser.HTMLParseError:
-                self.log.debug('Encountered a problem parsing %u.  Title may '
-                               'already be set, though', url)
-            if parser.title:
-                domain = utils.web.getDomain(fd.geturl()
+            (target, title) = r
+            if title:
+                domain = utils.web.getDomain(target
                         if self.registryValue('snarferShowTargetDomain', channel)
                         else url)
-                title = utils.web.htmlToText(parser.title.strip())
-                if sys.version_info[0] < 3:
-                    if isinstance(title, unicode):
-                        title = title.encode('utf8', 'replace')
                 s = format(_('Title: %s (at %s)'), title, domain)
                 irc.reply(s, prefixNick=False)
     titleSnarfer = urlSnarfer(titleSnarfer)
@@ -269,33 +285,15 @@ class Web(callbacks.PluginRegexp):
         if not self._checkURLWhitelist(url):
             irc.error("This url is not on the whitelist.")
             return
-        size = conf.supybot.protocols.http.peekSize()
-        text = utils.web.getUrl(url, size=size)
-        try:
-            text = text.decode(utils.web.getEncoding(text) or 'utf8',
-                    'replace')
-        except UnicodeDecodeError:
-            pass
-        parser = Title()
-        if sys.version_info[0] >= 3 and isinstance(text, bytes):
-            irc.error(_('Could not guess the page\'s encoding. (Try '
-                    'installing python-charade.)'), Raise=True)
-        try:
-            parser.feed(text)
-        except HTMLParser.HTMLParseError:
-            self.log.debug('Encountered a problem parsing %u.  Title may '
-                           'already be set, though', url)
-        if parser.title:
-            title = utils.web.htmlToText(parser.title.strip())
+        r = self.getTitle(irc, url, True)
+        if not r:
+            return
+        (target, title) = r
+        if title:
             if not [y for x,y in optlist if x == 'no-filter']:
                 for i in range(1, 4):
                     title = title.replace(chr(i), '')
             irc.reply(title)
-        elif len(text) < size:
-            irc.reply(_('That URL appears to have no HTML title.'))
-        else:
-            irc.reply(format(_('That URL appears to have no HTML title '
-                             'within the first %S.'), size))
     title = wrap(title, [getopts({'no-filter': ''}), 'httpUrl'])
 
     @internationalizeDocstring
