@@ -36,12 +36,14 @@ object (which, as you'll read later, is quite...full-featured :))
 """
 
 import re
-import sys
 import time
+import base64
+import datetime
 import functools
 
 from . import conf, ircutils, utils
 from .utils.iter import all
+from .utils import minisix
 
 ###
 # IrcMsg class -- used for representing IRC messages acquired from a network.
@@ -50,6 +52,37 @@ from .utils.iter import all
 class MalformedIrcMsg(ValueError):
     pass
 
+# http://ircv3.net/specs/core/message-tags-3.2.html#escaping-values
+SERVER_TAG_ESCAPE = [
+    ('\\', '\\\\'), # \ -> \\
+    (' ', r'\s'),
+    (';', r'\:'),
+    ('\r', r'\r'),
+    ('\n', r'\n'),
+    ]
+escape_server_tag_value = utils.str.MultipleReplacer(
+        dict(SERVER_TAG_ESCAPE))
+unescape_server_tag_value = utils.str.MultipleReplacer(
+        dict(map(lambda x:(x[1],x[0]), SERVER_TAG_ESCAPE)))
+
+def parse_server_tags(s):
+    server_tags = {}
+    for tag in s.split(';'):
+        if '=' not in tag:
+            server_tags[tag] = None
+        else:
+            (key, value) = tag.split('=', 1)
+            server_tags[key] = unescape_server_tag_value(value)
+    return server_tags
+def format_server_tags(server_tags):
+    parts = []
+    for (key, value) in server_tags.items():
+        if value is None:
+            parts.append(key)
+        else:
+            parts.append('%s=%s' % (key, escape_server_tag_value(value)))
+    return '@' + ';'.join(parts)
+
 class IrcMsg(object):
     """Class to represent an IRC message.
 
@@ -81,7 +114,8 @@ class IrcMsg(object):
     # data.  Goodbye, __slots__.
     # On second thought, let's use methods for tagging.
     __slots__ = ('args', 'command', 'host', 'nick', 'prefix', 'user',
-                 '_hash', '_str', '_repr', '_len', 'tags', 'reply_env')
+                 '_hash', '_str', '_repr', '_len', 'tags', 'reply_env',
+                 'server_tags', 'time')
     def __init__(self, s='', command='', args=(), prefix='', msg=None,
             reply_env=None):
         assert not (msg and s), 'IrcMsg.__init__ cannot accept both s and msg'
@@ -99,6 +133,11 @@ class IrcMsg(object):
                 if not s.endswith('\n'):
                     s += '\n'
                 self._str = s
+                if s[0] == '@':
+                    (server_tags, s) = s.split(' ', 1)
+                    self.server_tags = parse_server_tags(server_tags[1:])
+                else:
+                    self.server_tags = {}
                 if s[0] == ':':
                     self.prefix, s = s[1:].split(None, 1)
                 else:
@@ -110,6 +149,13 @@ class IrcMsg(object):
                 else:
                     self.args = s.split()
                 self.command = self.args.pop(0)
+                if 'time' in self.server_tags:
+                    s = self.server_tags['time']
+                    date = datetime.datetime.strptime(s, '%Y-%m-%dT%H:%M:%S.%fZ')
+                    date = minisix.make_datetime_utc(date)
+                    self.time = minisix.datetime__timestamp(date)
+                else:
+                    self.time = time.time()
             except (IndexError, ValueError):
                 raise MalformedIrcMsg(repr(originalString))
         else:
@@ -133,11 +179,15 @@ class IrcMsg(object):
                 else:
                     self.reply_env = None
                 self.tags = msg.tags.copy()
+                self.server_tags = msg.server_tags
+                self.time = msg.time
             else:
                 self.prefix = prefix
                 self.command = command
                 assert all(ircutils.isValidArgument, args), args
                 self.args = args
+                self.time = None
+                self.server_tags = {}
         self.args = tuple(self.args)
         if isUserHostmask(self.prefix):
             (self.nick,self.user,self.host)=ircutils.splitHostmask(self.prefix)
@@ -545,7 +595,7 @@ def kick(channel, nick, s='', prefix='', msg=None):
         assert isNick(nick), repr(nick)
     if msg and not prefix:
         prefix = msg.prefix
-    if sys.version_info[0] < 3 and isinstance(s, unicode):
+    if minisix.PY2 and isinstance(s, unicode):
         s = s.encode('utf8')
     assert isinstance(s, str)
     if s:
@@ -565,7 +615,7 @@ def kicks(channels, nicks, s='', prefix='', msg=None):
         assert areNicks(nicks), repr(nicks)
     if msg and not prefix:
         prefix = msg.prefix
-    if sys.version_info[0] < 3 and isinstance(s, unicode):
+    if minisix.PY2 and isinstance(s, unicode):
         s = s.encode('utf8')
     assert isinstance(s, str)
     if s:
@@ -582,7 +632,7 @@ def privmsg(recipient, s, prefix='', msg=None):
     if conf.supybot.protocols.irc.strictRfc():
         assert (areReceivers(recipient)), repr(recipient)
         assert s, 's must not be empty.'
-    if sys.version_info[0] < 3 and isinstance(s, unicode):
+    if minisix.PY2 and isinstance(s, unicode):
         s = s.encode('utf8')
     assert isinstance(s, str)
     if msg and not prefix:
@@ -614,7 +664,7 @@ def notice(recipient, s, prefix='', msg=None):
     if conf.supybot.protocols.irc.strictRfc():
         assert areReceivers(recipient), repr(recipient)
         assert s, 'msg must not be empty.'
-    if sys.version_info[0] < 3 and isinstance(s, unicode):
+    if minisix.PY2 and isinstance(s, unicode):
         s = s.encode('utf8')
     assert isinstance(s, str)
     if msg and not prefix:
@@ -664,7 +714,7 @@ def part(channel, s='', prefix='', msg=None):
         assert isChannel(channel), repr(channel)
     if msg and not prefix:
         prefix = msg.prefix
-    if sys.version_info[0] < 3 and isinstance(s, unicode):
+    if minisix.PY2 and isinstance(s, unicode):
         s = s.encode('utf8')
     assert isinstance(s, str)
     if s:
@@ -680,7 +730,7 @@ def parts(channels, s='', prefix='', msg=None):
         assert all(isChannel, channels), channels
     if msg and not prefix:
         prefix = msg.prefix
-    if sys.version_info[0] < 3 and isinstance(s, unicode):
+    if minisix.PY2 and isinstance(s, unicode):
         s = s.encode('utf8')
     assert isinstance(s, str)
     if s:
@@ -709,7 +759,7 @@ def topic(channel, topic=None, prefix='', msg=None):
         return IrcMsg(prefix=prefix, command='TOPIC',
                       args=(channel,), msg=msg)
     else:
-        if sys.version_info[0] < 3 and isinstance(topic, unicode):
+        if minisix.PY2 and isinstance(topic, unicode):
             topic = topic.encode('utf8')
         assert isinstance(topic, str)
         return IrcMsg(prefix=prefix, command='TOPIC',
@@ -772,7 +822,7 @@ def names(channel=None, prefix='', msg=None):
 def mode(channel, args=(), prefix='', msg=None):
     if msg and not prefix:
         prefix = msg.prefix
-    if isinstance(args, basestring):
+    if isinstance(args, minisix.string_types):
         args = (args,)
     else:
         args = tuple(map(str, args))
@@ -818,6 +868,21 @@ def ison(nick, prefix='', msg=None):
         prefix = msg.prefix
     return IrcMsg(prefix=prefix, command='ISON', args=(nick,), msg=msg)
 
+def monitor(subcommand, nicks=None, prefix='', msg=None):
+    if conf.supybot.protocols.irc.strictRfc():
+        for nick in nicks:
+            assert isNick(nick), repr(nick)
+        assert subcommand in '+-CLS'
+        if subcommand in 'CLS':
+            assert nicks is None
+    if msg and not prefix:
+        prefix = msg.prefix
+    if not isinstance(nicks, str):
+        nicks = ','.join(nicks)
+    return IrcMsg(prefix=prefix, command='MONITOR', args=(subcommand, nicks),
+            msg=msg)
+
+
 def error(s, msg=None):
     return IrcMsg(command='ERROR', args=(s,), msg=msg)
 
